'****************************************************************
'*  Name    : SnapShotDemo.bas                                  *
'*  Author  : George Towler                                     *
'*  Date    : see History                                       *
'*  Version : see VersionHigh/VersionLow                        *
'*  Compiler: 3.6.2.8                                           *
'*  Notes   : Demonstrator For SnapShot.inc                     *
'*          : see SnapShot.inc for usage notes                  *
'*          :                                                   *
'*          : LVP may not be used on a device that has          *
'*          :     previously been programmed with HVP or that   *
'*          :     does not support it.                          *
'*          :                                                   *
'****************************************************************
'History
' 1.0  06/01/18  removed any reference to D_DumpingLED
'                replaced D_UseHardSerial with D_SerialNum = 0
' 0.9  29/12/18  Added options to allow LVP
'                HexTag position changed, HexTag found by Label not position
'                Heap demo removed, it's bank switching is fucked in 3.6.1.6
'                Assembler macros replaced with preprocessor equivalents (for 18F26K42 and  18F27K42)
'                added D_LimitEEPROM to bypass problems with word addresses on EEPROM for 18F26K42 and 18F27K42
' 0.8  26/05/18  extra baud rates added for bit bang
'                added D_SendPort option
'                added D_HardSerialNum option for PIC with multiple UARTS
'                added config for 18F25K50, 18F26K22 and 18F26K42 (18F26K42 only partial functional)
' 0.7  01/10/17  IDLOCS replaced with ROM based SourceID
' 0.6  28/09/17  Documentation

  'Device = 16F690
  'Device = 16F886
  'Device = 18F2550
  'Device = 18F4550           ' nonsense for test 1
  'Device = 18F14K50          ' nonsense For test 2
  'Device = 18F25K50
  'Device = 18F26K22
  'Device = 18F27K40          ' nonsense for test 3
  Device = 18F27K42           ' nonsense for test 4

  ' if you are adding a new device you need to check the comms section in SnapShot.inc
  $if _device = _18F2550 Or _device = _18F4550 Or _device = _18F14K50 Or _device = _18F25K50
    Xtal = 48
  $elseif _device = _18F27K42 Or _device = _18F26K22 Or _device = _18F27K40
    Xtal = 64
  $else
    Xtal = 8
  $endif

  $define VersionHigh 1                 ' application version high (4 bits so must not exceed 15)
  $define VersionLow 0                  ' application version low  (4 bits so must not exceed 15)
  $define EnableSnapShot True           ' enable SnapShot

  $if _device = _18F27K42 Or _device = _18F27K40
    $define Use_LVP True                ' True = enable LVP for NSDSP. See note in header
  $else
    $define Use_LVP False               ' False = disable LVP for PicKit2
  $endif

  '****************** Configuration *******************************
  ' You'll want to include some or all of these definitions in the code under test
  $define D_SerialNum 2                 ' 0, 1 or 2 - 0 = bit bang else UART number
  $define D_BaudRate 115200             ' desired baud rate
  $define D_UseCRC True                 ' True = compute and use CRC - code size and transaction time reduction for F16
  $define D_CheckNulls True             ' True = check for contiguous nulls - Code size And transaction time reduction For F16
  $define D_SendEEPROM True             ' True = send EEPROM data
  $define D_LimitEEPROM True            ' True = limit EEPROM to 256 (ie pointer is byte not word)
  $define D_SendPort True               ' True = send Port data
  $define D_SendSFR False               ' True = send SFR data
  '*****************************************************************

  ' a guess at devices using a crystal
  $if _xtal = 48 Or _xtal = 20
    $if _device = _18F2550 Or _device = _18F4550
      Symbol OSC2 = PORTA.6             ' no OSC1 because it is a dedicated pin
    $elseif _device = _18F14K50
      Symbol OSC1 = PORTA.5
      Symbol OSC2 = PORTA.4
    $endif
  $endif

  Declare All_Digital = True
  Declare Reminders = On
  Declare Show_System_Variables = On
  Declare Optimiser_Level = 0           ' no optimise on 16F
  Declare Dead_Code_Remove = Off        ' real men don't write dead code
  Declare Bootloader = Off
  Declare Create_Coff = False           ' create a COFF file
  Declare Keep_Hex_File = False         ' do not deletes HEX fle on failed compiles - for Auto Import Hex option on PicKit2
  Declare MemWrite_Int_Control = True   ' conrtol interrupts during EEPROM write
  Declare Hints = True                  ' control compiler hints
  Declare Stack_Size = 20               ' stack for pseudo proceedure call/return
  'Declare Onboard_USB = False          ' frees up memory if USB not used
  'Declare TRIS_Reverse = On             ' on = TRSC first then Port/LAT High/Low

  $if _device = _18F2550 Or _device = _18F4550
    TRISA  = %11111001    '            A6=OSC       A5=       A4=          A3=           A2=Aux        A1=Sel        A0=
    TRISB  = %11111101    ' B7=PGD     B6=PGC       B5=       B4=           B3=           B2=           B1=SCK/SCL    B0=
    TRISC  = %10111111    ' C7=RX/SDO  C6=TX        C5=D+     C4=D-         C3=Vusb       C2=           C1=           C0=
  $elseif _device = _18F14K50
    TRISA  = %11111111    '                                                                             A1=PGC/D-     A0=PGD/D+
    TRISB  = %00111111    ' B7=TX      B6=SCL/SCK   B5=RX     B4=SCI/SDA
    TRISC  = %01111001    ' C7=SDO     C6=          C5=       C4=D-         C3=           C2=Aux        C1=Sel        C0=
  $elseif _device = _18F25K50
    TRISA  = %11111001    '            A6=          A5=       A4=           A3=           A2=Aux        A1=Sel        A0=
    TRISB  = %11110101    ' B7=PGD     B6=PGC       B5=       B4=           B3=SDO        B2=           B1=SCK/SCL    B0=SDI/SDA
    TRISC  = %10111111    ' C7=RX      C6=TX        C5=D+     C4=D-         C3=Vusb       C2=           C1=           C0=
  $elseif _device = _18F26K22
    TRISA  = %11111001    ' A7=         A6=         A5=       A4=           A3=           A2=Aux        A1=Sel        A0=
    TRISB  = %10110101    ' B7=PGD/RX2  B6=PGC/TX2  B5=       B4=           B3=SDO2       B2=SDI2/SDA2  B1=SCK2/SCL2  B0=
    TRISC  = %10110111    ' C7=RX1      C6=TX1      C5=       C4=SDI1/SDA1  C3=SCK1/SCL1  C2=           C1=           C0=
  $elseif _device = _18F27K42 Or _device = _18F27K40
    TRISA  = %11111001    ' A7=         A6=         A5=       A4=           A3=           A2=Aux        A1=Sel        A0=
    TRISB  = %11110101    ' B7=PGD      B6=PGC      B5=       B4=           B3=SDO2       B2=SDI2/SDA2  B1=SCK2/SCL2  B0=
    TRISC  = %10110101    ' C7=RX1      C6=TX1      C5=       C4=SDI1/SDA1  C3=SCK1/SCL1  C2=           C1=TX2        C0=RX2
  $elseif _device = _16F886
    TRISA  = %11111001    '             A6=OSC      A5=       A4=           A3=           A2=Aux        A1=Sel        A0=
    TRISB  = %11111111    ' B7=PGD      B6=PGC      B5=       B4=           B3=           B2=           B1=           B0=
    TRISC  = %10010111    ' C7=RX1      C6=TX1      C5=SDO    C4=SDI/SDA    C3=SCK/SCL    C2=           C1=           C0=OSC
  $elseif _device = _16F690
    TRISA  = %11111111    '                         A5=       A4=           A3=           A2=           A1=PGC        A0=PGD
    TRISB  = %00111111    ' B7=TX       B6=SCL/SCK  B5=RX     B4=SDI/SDA
    TRISC  = %01111111    ' C7=SDO      C6=         C5=       C4=           C3=           C2=Aux        C1=Sel        C0=
  $endif

  $if _device = _16F690 Or _device = _18F14K50
    Symbol Rx  = PORTB.5                ' equals sign optional
    Symbol Tx  = PORTB.7
  $elseif _device = _18F26K22
    Symbol Rx1  = PORTC.7                ' equals sign optional
    Symbol Tx1  = PORTC.6
    Symbol Rx2  = PORTB.7                ' equals sign optional
    Symbol Tx2  = PORTB.6
  $elseif _device = _18F27K42 Or _device = _18F27K40
    Symbol Rx1  = PORTC.7                ' equals sign optional
    Symbol Tx1  = PORTC.6
    Symbol Rx2  = PORTC.0                ' equals sign optional
    Symbol Tx2  = PORTC.1
  $else
    Symbol Rx   = PORTC.7                ' best quess for unspecified device
    Symbol Tx   = PORTC.6
  $endif

  $if _device = _16F690 Or _device = _18F14K50
    Symbol Aux = PORTC.2
    Symbol Sel = PORTC.1
  $else
    Symbol Aux = PORTA.2
    Symbol Sel = PORTA.1
  $endif

  Dim YASByte_Var7          As SByte                  ' semi-colon as comment indicator

  Dim EepromCounter         As Byte                   ' byte to a byte address works
  $if D_LimitEEPROM = True
    Dim EepromAddress       As Byte
  $else
    Dim EepromAddress       As Word
  $endif

  Dim SByte_Array_Var[10]   As SByte                  ' this is an signed byte array
  Dim SWord_Array_Var[5]    As SWord                  '
  Dim BigArray[18]          As Byte                   ' SWord_Array_Var had no comment just a inverted comma
  Dim dummy1                As Byte                   ' Short
  Dim dummy2                As Byte                   ' LongLongLongLongLongLongLongLongLongLongLongLongLongLongLongLongLongLong
  Dim dummy3                As Byte                   ' some variable
  Dim Silly_Array[1]        As Byte
  Dim YABit_Var7            As Bit
  Dim Bit_Var               As Bit
  Dim Bit_Test              As Bit
  Dim Byte_Var              As Byte
  Dim Word_Var              As Word
  Dim Word_Var_Lo           As Word_Var.LowByte
  Dim DWord_Var             As Dword
  Dim Sbyte_Var             As SByte
  Dim SWord_Var             As SWord
  Dim SDWord_Var            As SDword
  Dim Byte_Array_Var[8]     As Byte
  Dim xbank_float           As Float
  Dim Float_Var0            As Float
  Dim Float_Var1            As Float
  Dim Float_Var2            As Float
  Dim Float_Var3            As Float
  Dim Float_Var4            As Float
  Dim Float_Var5            As Float
  Dim Float_Var6            As Float
  Dim Float_Var7            As Float
  Dim Float_Var8            As Float
  Dim Float_Var9            As Float
  Dim Float_Var10           As Float
  Dim MyCounter             As Byte
  Dim Dummy                 As Byte
  Dim YAByte_Var            As Byte
  Dim YAsByte_Array_Var[11] As SByte
  Dim YAWord_Var            As Word
  Dim YAWord_Var_Hi         As YAWord_Var.HighByte
  Dim YAByte_Array_Var[10]  As Byte

  $if _core = 16
    Dim Zero_Var[25]        As Byte
    Dim Float_Array_Var[2]  As Float
    Dim CodeArray1          As Code = 1, 2, 3, 4      ' problematic for ident on 16F
    Dim CodeArray3          As Code = "Hello"
    'Dim String_Var          As String * 230           ' inverted comma followed by ;semi-colon
    Dim String_Var[230]     As Byte
    Dim Mirror_var          As Byte At YASByte_Var7   ; semi-colon followed by ' inverted comma
    Dim dummy4              As Byte At dummy1
  $endif

  Clear

'*************** oscilator setup if required *********************
  $if _device = _16F690 Or _device = _16F886
    OSCCON = %01110000  ' 8MHz clock

  $elseif _device = _18F25K50
    Symbol PLLEN = OSCCON2.4           ' software PLL Enable Bit
    Symbol SPLLMULT = OSCTUNE.7        ' software PLL multiplier select bit
    Symbol PLLRDY = OSCCON2.7          ' PLL Run Status Bit

    PLLEN = 0                          ' turn off PLL for safety while we setup clock
    SPLLMULT = 1                       ' SPLLMULT 3xPLL
    OSCCON = %01110000                 ' Primary OSC 48Mhz (16MHz + 3xPLL)
    OSCCON2 = %00010000                ' enable the PLL
    ACTCON = %10010000                 ' active tuning for USB
    While PLLRDY = 0
      DelayMS 1                        ' wait a bit for the clock to stabilize
    Wend

  $elseif _device = _18F26K22
    Symbol PLLEN = OSCTUNE.6           ' software PLL Enable Bit
    Symbol PLLRDY = OSCCON2.7          ' PLL Run Status Bit

    PLLEN = 0                          ' turn off PLL for safety while we setup clock
    OSCCON = %01111000                 ' Oscillator Control Register
    OSCCON2 = %00000000                ' Oscillator Control Register 2
    OSCTUNE= %11000000                 ' Oscillator Tuning Register
    While PLLRDY = 0
      DelayMS 1                        ' wait a bit for the clock to stabilize
    Wend

  $elseif _device = _18F27K42
    ADCON0 = %00000000                 ' Disable ADC
    OSCCON1 = %01100000                ' Primary OSC 64Mhz
    OSCFRQ =  %00001000                ' 64MHz

    PPSLOCK   = $55                    ' PPS unlock
    PPSLOCK   = $AA
    PPSLOCK.0 = 0
    U1RXPPS   = 0x17                   ' UART1 RC7 > RX1
    RC6PPS    = 0x13                   '       TX1 > RC6
    U2RXPPS   = 0x10                   ' UART2 RC0 > RX2
    RC1PPS    = 0x16                   '       TX2 > RC1
    PPSLOCK   = $55
    PPSLOCK   = $AA
    PPSLOCK.0 = 1                      ' PPS LOCK

  $elseif _device = _18F27K40
    Symbol _PPS_FN_RX2 = 12            ' workaround
    ADCON0 = %00000000                 ' Disable ADC
    OSCCON1 = %01100000                ' Primary OSC 64Mhz (16MHz + 4xPLL)
    OSCFRQ =  %00001000                ' 64MHz

    PPSLOCK = $55                      ' PPS unlock
    PPSLOCK = $AA
    PPSLOCK.0 = 0
    RX1PPS = 0x0017                    ' UART1 RC7 > RX1
    RC6PPS = 0x0009                    '       TX1 > RC6
    RX2PPS = 0x0010                    ' UART2 RC0 > RX2
    RC1PPS = 0x000B                    '       TX2 > RC1
    PPSLOCK = $55
    PPSLOCK = $AA
    PPSLOCK.0 = 1                      ' PPS LOCK

  $endif
'*****************************************************************


'*****************************************************************
  $if EnableSnapShot = True
    Include "SnapShot.inc"                  ' <--------- MUST be inserted into code under test
    D_AddressStart = AddressOf(YASByte_Var7)    ' <--------- MAY be inserted into code under test
  $else
    $define Dump(pValue) '
    Nop
    $define DumpW(pValue) '
    Nop
  $endif
'*****************************************************************


  DumpW(12)                          ' Reset

MAIN:

  'Clear                             ' if D_AddressStart is specified this will overwrite it - if clear nec. insert before include

  BigArray[0] = $55
  bigarray#17 = $CC
  DelayMS 2000                       ' nice big delay
  Word_Var = 1
  MyCounter = 2

  EepromAddress = $01
  EepromCounter = $55


DumpLoop:

  DelayMS 2000                       ' nice big delay

' initialise Demo variables

  xbank_float = 111.111
  dummy1 = 10
  dummy3 = 15
  YASByte_Var7 = -128
  Bit_Var = 1
  YABit_Var7 = 0
  Byte_Var = 1
  Byte_Array_Var#0 = "T"
  Byte_Array_Var[1] = "h"
  Byte_Array_Var[2] = "e"
  Byte_Array_Var[3] = " "
  Byte_Array_Var[4] = "Q"
  Byte_Array_Var[5] = "u"
  Byte_Array_Var[6] = "i"
  DWord_Var = 127879
  Sbyte_Var = 65
  SWord_Var = -345
  SDWord_Var = -45767
  Float_Var0 = 0
  Float_Var1 = 1
  Float_Var2 = -1
  Float_Var3 = 3.1421567
  Float_Var4 = -3.1421567

  $if _core = 16
    Float_Array_Var[0] = 3.1421567
    Float_Array_Var#1 = -3.1421567
    'String_Var = "abcdef"
    Str String_Var = "abcdef"
    dummy4 = 20
  $endif

  Float_Var5 = 100
  Float_Var6 = -100
  Float_Var7 = 1000
  Float_Var8 = -1000
  Float_Var9 = 10000
  Float_Var10 = -10000
  SByte_Array_Var[1] = 34
  SByte_Array_Var[2] = 100
  SByte_Array_Var[3] = -100
  SByte_Array_Var[4] = 127
  SWord_Array_Var[0] = 56
  SWord_Array_Var[1] = -275
  SWord_Array_Var[2] = 156
  Sword_Array_Var#3 = -32768
  Sword_Array_Var#4 = 32767

  PORTC = $FF

  Set Sel
  DumpW(253)                          ' Variable initialized
  Clear Sel

  Inc MyCounter
  Float_Var0 = 1
  PORTC = $55
  BigArray#3 = "e"
  DelayMS 2000                       ' nice big delay
  Byte_Array_Var#0 = "X"
  Byte_Array_Var[1] = "y"

  Set Sel
  DumpW(2)                            ' Some other nonsense here
  Clear Sel

  Inc MyCounter
  Bit_Var = 0
  $if _core = 16
    String_Var = "some strin"
  $endif
  SByte_Array_Var[0] = -1
  DelayMS 2000                       ' nice big delay
  Inc dummy1

  Set Sel
  DumpW(3)                            ' wrote to string
  Clear Sel

  Inc MyCounter
  Bit_Var = 1
  YABit_Var7 = 1
  BigArray#3 = "E"
  SWord_Array_Var[0] = -1
  DelayMS 2000                       ' nice big delay
  $if _core = 16
    Inc dummy4
  $endif

  dummy3 = 12
  Set Sel
  DumpW(4)                            ' some other stuff completed
  Clear Sel

  $if _core = 16
    Float_Array_Var[0] = -333.333
    Float_Array_Var#1 = 222.222
  $endif

  EWrite EepromAddress, [EepromCounter]       ' <-------- 18F27K42 works if both in Bank 0 and EepromCounter is a byte
  Inc EepromAddress
  Inc EepromCounter

  Inc Word_Var
  Set Sel
  Dump(255)                           ' EEPROM write done
  Clear Sel

  DelayMS 2000

  Inc MyCounter

  Set Sel
  DumpW(6)                            ' all done, ready to loop back
  Clear Sel

  Include "TestInclude.inc"

  Inc MyCounter

  DelayMS 2000
  GoTo DumpLoop

'**************************************************************************************
  ' 20MHz crystal
  $if _device = _18F2550 Or _device = _18F4550
    Config_Start
      PLLDIV = 5          ;Divide by 5 (20 MHz oscillator INPUT)
      CPUDIV = OSC1_PLL2  ;[Primary Oscillator Src: /1][96 MHz PLL Src: /2]
      USBDIV = 2          ;USB clock SOURCE comes FROM the 96 MHz PLL divided by 2
      FOSC = HSPLL_HS     ;HS oscillator, PLL enabled (HSPLL)
      FCMEN = OFF         ;Fail-Safe Clock Monitor disabled
      IESO = OFF          ;Oscillator Switchover mode disabled
      PWRT = On           ;PWRT enabled
      BORV = 3            ;Minimum setting
      VREGEN = OFF        ;USB voltage regulator disabled   ******** NOTE **********
      WDT = OFF           ;WDT disabled (controled by the SWDTEN bit)
      WDTPS = 32768       ;1:32768
      CCP2MX = On         ;CCP2 INPUT/OUTPUT IS multiplexed with RC1
      PBADEN = OFF        ;PORTB<4:0> pins are configured AS digital I/O on reset
      LPT1OSC = On        ;Timer1 configured FOR low-power operation
      STVREN = On         ;Stack FULL/underflow will cause reset

      $if Use_LVP = True
        MCLRE = On        ;RE3 INPUT pin enabled; MCLR pin disabled
        LVP = On          ;SINGLE-Supply ICSP disabled
        BOR = OFF          ;Brown-out Reset disabled in hardware and software
      $else
        MCLRE = OFF       ;RE3 INPUT pin enabled; MCLR pin disabled
        LVP = OFF         ;SINGLE-Supply ICSP disabled
        BOR = On          ;Brown-out Reset enabled in hardware only (SBOREN is disabled)
      $endif

      $if _device = _18F4550
        ICPRT = OFF       ;ICPORT disabled
      $endif

      XINST = OFF         ;Instruction SET extension AND Indexed Addressing MODE disabled (Legacy MODE)
      Debug = OFF         ;Background debugger disabled, RB6 AND RB7 configured AS general purpose I/O pins
      Cp0 = OFF           ;Block 0 (000800-001FFFh) is not CODE-protected
      CP1 = OFF           ;Block 1 (002000-003FFFh) is not CODE-protected
      CP2 = OFF           ;Block 2 (004000-005FFFh) is not CODE-protected
      CP3 = OFF           ;Block 3 (006000-007FFFh) is not CODE-protected
      CPB = OFF           ;Boot block (000000-0007FFh) is not CODE-protected
      CPD = OFF           ;DATA EEPROM is not code-protected
      WRT0 = OFF          ;Block 0 (000800-001FFFh) is not WRITE-protected
      WRT1 = OFF          ;Block 1 (002000-003FFFh) is not WRITE-protected
      WRT2 = OFF          ;Block 2 (004000-005FFFh) is not WRITE-protected
      WRT3 = OFF          ;Block 3 (006000-007FFFh) is not WRITE-protected
      WRTC = OFF          ;Configuration registers (300000-3000FFh) are NOT WRITE-protected
      WRTB = OFF          ;Boot block (000000-0007FFh) is not WRITE-protected
      WRTD = OFF          ;DATA EEPROM is not write-protected
      EBTR0 = OFF         ;Block 0 (000800-001FFFh) is not protected FROM table reads executed IN other blocks
      EBTR1 = OFF         ;Block 1 (002000-003FFFh) is not protected FROM table reads executed IN other blocks
      EBTR2 = OFF         ;Block 2 (004000-005FFFh) is not protected FROM table reads executed IN other blocks
      EBTR3 = OFF         ;Block 3 (006000-007FFFh) is not protected FROM table reads executed IN other blocks
      EBTRB = OFF         ;Boot block (000000-0007FFh) is not protected FROM table reads executed IN other blocks
    Config_End
  $endif

  ' 12 MHz crystal
  $if _device = _18F14K50
    Config_Start
      CPUDIV = NOCLKDIV  ;No CPU System Clock divide
      USBDIV = OFF      ;USB clock comes directly from the OSC1/OSC2 oscillator block; no divide
      FOSC = HS          ;HS oscillator
      PLLEN = On        ;Oscillator multiplied by 4
      PCLKEN = On        ;Primary clock enabled
      FCMEN = OFF        ;Fail-Safe Clock Monitor disabled
      IESO = OFF        ;Oscillator Switchover mode disabled
      PWRTEN = On        ;PWRT enabled
      BORV = 19          ;VBOR set to 1.9 V nominal
      WDTEN = OFF        ;WDT is controlled by SWDTEN bit of the WDTCON register
      WDTPS = 32768      ;1:32768
      HFOFST = On        ;HFINTOSC starts clocking the CPU without waiting for the oscillator to stablize.
      $if Use_LVP = True
        MCLRE = On      ;RE3 INPUT pin enabled; MCLR pin disabled
        LVP = On        ;SINGLE-Supply ICSP disabled
        BOREN = OFF      ;Brown-out Reset disabled in hardware and software
      $else
        MCLRE = OFF     ;RE3 INPUT pin enabled; MCLR pin disabled
        LVP = OFF       ;SINGLE-Supply ICSP disabled
        BOREN = On      ;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
      $endif

      STVREN = On        ;Stack full/underflow will cause Reset
      BBSIZ = OFF        ;1kW boot block size
      XINST = OFF        ;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
      Debug = OFF        ;Background debugger disabled, RA0 and RA1 configured as general purpose I/O pins
      Cp0 = OFF          ;Block 0 not code-protected
      CP1 = OFF          ;Block 1 not code-protected
      CPB = OFF          ;Boot block not code-protected
      CPD = OFF          ;Data EEPROM not code-protected
      WRT0 = OFF         ;Block 0 not write-protected
      WRT1 = OFF         ;Block 1 not write-protected
      WRTC = OFF         ;Configuration registers not write-protected
      WRTB = OFF         ;Boot block not write-protected
      WRTD = OFF         ;Data EEPROM not write-protected
      EBTR0 = OFF        ;Block 0 not protected from table reads executed in other blocks
      EBTR1 = OFF        ;Block 1 not protected from table reads executed in other blocks
      EBTRB = OFF        ;Boot block not protected from table reads executed in other blocks
    Config_End
  $endif

  ' internal clock 8MHz
  $if _device = _16F690
    Config FOSC_INTRCIO, WDTE_OFF, PWRTE_ON, MCLRE_OFF, CP_OFF, CPD_OFF, BOREN_ON, IESO_OFF, FCMEN_ON
    $if Use_LVP = True
      $error "This device is not able to utilize LVP"
    $endif
  $endif

  ' internal clock 8MHz
  $if _device = _16F886
    $if Use_LVP = True
      Config1 FOSC_INTRC_NOCLKOUT, WDTE_OFF, PWRTE_ON, MCLRE_ON, CP_OFF, CPD_OFF, BOREN_OFF, IESO_OFF, FCMEN_OFF, LVP_ON, DEBUG_OFF
    $else
      Config1 FOSC_INTRC_NOCLKOUT, WDTE_OFF, PWRTE_ON, MCLRE_OFF, CP_OFF, CPD_OFF, BOREN_SBODEN, IESO_OFF, FCMEN_OFF, LVP_OFF, DEBUG_OFF
  $endif
    Config2 BOR4V_BOR21V, WRT_OFF
  $endif

  ' 48Hhz without a crystal
  $if _device = _18F25K50
    Config_Start
      PLLSEL = PLL3X    ;3x clock multiplier
      CFGPLLEN = OFF    ;PLL Disabled (firmware controlled)
      CPUDIV = NOCLKDIV  ;CPU uses system clock (no divide)
      LS48MHZ = SYS48X8  ;System clock at 48 MHz, USB clock divider is set to 8
      FOSC = INTOSCIO    ;Internal oscillator
      PCLKEN = OFF      ;Primary oscillator shutdown firmware controlled
      FCMEN = OFF        ;Fail-Safe Clock Monitor disabled
      IESO = OFF        ;Oscillator Switchover mode disabled
      nPWRTEN = On      ;Power up timer enabled
      BORV = 285        ;BOR set to 2.85V nominal
      nLPBOR = OFF      ;Low-Power Brown-out Reset disabled
      WDTEN = SWON      ;WDT controlled by firmware (SWDTEN enabled)
      WDTPS = 32768      ;1:32768
      CCP2MX = RC1      ;CCP2 input/output is multiplexed with RC1
      PBADEN = OFF      ;PORTB<5:0> pins are configured as digital I/O on Reset
      T3CMX = RC0        ;T3CKI function is on RC0
      SDOMX = RB3        ;SDO function is on RB3
      STVREN = On        ;Stack full/underflow will cause Reset
      ICPRT = OFF        ;ICPORT disabled
      XINST = OFF        ;Instruction set extension and Indexed Addressing mode disabled

      $if Use_LVP = True
        MCLRE = On      ;RE3 INPUT pin enabled; MCLR pin disabled
        LVP = On        ;SINGLE-Supply ICSP disabled
        BOREN = OFF      ;Brown-out Reset disabled in hardware and software
      $else
        MCLRE = OFF     ;RE3 INPUT pin enabled; MCLR pin disabled
        LVP = OFF       ;SINGLE-Supply ICSP disabled
        BOREN = On      ;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
      $endif

      Debug = OFF        ;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
      Cp0 = OFF          ;Block 0 is not code-protected
      CP1 = OFF          ;Block 1 is not code-protected
      CP2 = OFF          ;Block 2 is not code-protected
      CP3 = OFF          ;Block 3 is not code-protected
      CPB = OFF          ;Boot block is not code-protected
      CPD = OFF          ;Data EEPROM is not code-protected
      WRT0 = OFF        ;Block 0 (0800-1FFFh) is not write-protected
      WRT1 = OFF        ;Block 1 (2000-3FFFh) is not write-protected
      WRT2 = OFF        ;Block 2 (04000-5FFFh) is not write-protected
      WRT3 = OFF        ;Block 3 (06000-7FFFh) is not write-protected
      WRTC = OFF        ;Configuration registers (300000-3000FFh) are not write-protected
      WRTB = OFF        ;Boot block (0000-7FFh) is not write-protected
      WRTD = OFF        ;Data EEPROM is not write-protected
      EBTR0 = OFF        ;Block 0 is not protected from table reads executed in other blocks
      EBTR1 = OFF        ;Block 1 is not protected from table reads executed in other blocks
      EBTR2 = OFF        ;Block 2 is not protected from table reads executed in other blocks
      EBTR3 = OFF        ;Block 3 is not protected from table reads executed in other blocks
      EBTRB = OFF        ;Boot block is not protected from table reads executed in other blocks
    Config_End
  $endif

  ' internal clock 16MHz with 4xPLL = 64MHz
  $if _device = _18F26K22
    Config_Start
      FOSC = INTIO67     ; Internal oscillator block
      PLLCFG = On         ; Oscillator multiplied by 4
      PRICLKEN = On       ; Primary clock enabled
      FCMEN = OFF         ; Fail-Safe Clock Monitor disabled
      IESO = OFF         ; Oscillator Switchover mode disabled
      PWRTEN = On         ; Power up timer enabled
      BORV = 285         ; VBOR set to 2.85 V nominal
      WDTEN = OFF         ; Watch dog timer is always disabled. SWDTEN has no effect.
      WDTPS = 32768       ; 1:32768
      CCP2MX = PORTC1     ; CCP2 input/output is multiplexed with RC1
      PBADEN = OFF       ; PORTB<5:0> pins are configured as digital I/O on Reset
      CCP3MX = PORTB5     ; P3A/CCP3 input/output is multiplexed with RB5
      HFOFST = On         ; HFINTOSC output and ready status are not delayed by the oscillator stable status
      T3CMX = PORTC0     ; T3CKI is on RC0
      P2BMX = PORTB5     ; P2B is on RB5
      STVREN = On         ; Stack full/underflow will cause Reset
      XINST = OFF         ; Instruction set extension and Indexed Addressing mode disabled (Legacy mode)

      $if Use_LVP = True
        MCLRE = EXTMCLR   ; MCLR pin enabled, RE3 input pin disabled
        LVP = On         ; SINGLE-Supply ICSP disabled
        BOREN = OFF       ; Brown-out Reset disabled in hardware and software
      $else
        MCLRE = INTMCLR   ; RE3 input pin enabled; MCLR disabled
        LVP = OFF        ; SINGLE-Supply ICSP disabled
        BOREN = On       ; Brown-out Reset enabled and controlled by software (SBOREN is enabled)
      $endif

      Debug = OFF         ; Disabled
      Cp0 = OFF           ; Block 0 (000800-003FFFh) not code-protected
      CP1 = OFF           ; Block 1 (004000-007FFFh) not code-protected
      CP2 = OFF           ; Block 2 (008000-00BFFFh) not code-protected
      CP3 = OFF           ; Block 3 (00C000-00FFFFh) not code-protected
      CPB = OFF           ; Boot block (000000-0007FFh) not code-protected
      CPD = OFF           ; Data EEPROM not code-protected
      WRT0 = OFF         ; Block 0 (000800-003FFFh) not write-protected
      WRT1 = OFF         ; Block 1 (004000-007FFFh) not write-protected
      WRT2 = OFF         ; Block 2 (008000-00BFFFh) not write-protected
      WRT3 = OFF         ; Block 3 (00C000-00FFFFh) not write-protected
      WRTC = OFF         ; Configuration registers (300000-3000FFh) not write-protected
      WRTB = OFF         ; Boot Block (000000-0007FFh) not write-protected
      WRTD = OFF         ; Data EEPROM not write-protected
      EBTR0 = OFF         ; Block 0 (000800-003FFFh) not protected from table reads executed in other blocks
      EBTR1 = OFF         ; Block 1 (004000-007FFFh) not protected from table reads executed in other blocks
      EBTR2 = OFF         ; Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks
      EBTR3 = OFF         ; Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks
      EBTRB = OFF         ; Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
    Config_End
  $endif

  ' internal clock 64MHz
  $if _device = _18F27K42
    Config_Start
      FEXTOSC = OFF             ; Oscillator not enabled
      RSTOSC = HFINTOSC_64MHZ   ; HFINTOSC with HFFRQ = 64 MHz and CDIV = 1:1
      CLKOUTEN = OFF           ; CLKOUT function is disabled
      PR1WAY = OFF             ; PRLOCK bit can be set and cleared repeatedly
      CSWEN = On               ; Writing to NOSC and NDIV is allowed
      FCMEN = OFF               ; Fail-Safe Clock Monitor disabled
      PWRTS = PWRT_1           ; PWRT set at 1ms
      MVECEN = OFF             ; Interrupt contoller does not use vector table to prioritze interrupts
      IVT1WAY = OFF             ; IVTLOCK bit can be cleared and set repeatedly
      LPBOREN = On             ; ULPBOR enabled
      BORV = VBOR_2P7           ; Brown-out Reset Voltage (VBOR) set to 2.7V
      ZCD = OFF                 ; ZCD disabled. ZCD can be enabled by setting the ZCDSEN bit of ZCDCON
      PPS1WAY = OFF             ; PPSLOCK bit can be set and cleared repeatedly (subject to the unlock sequence)
      STVREN = On               ; Stack full/underflow will cause Reset

      $if Use_LVP = True
        MCLRE = EXTMCLR         ; MCLR pin enabled, RE3 input pin disabled
        LVP = On               ; SINGLE-Supply ICSP enabled
        BOREN = OFF             ; Brown-out Reset disabled in hardware and software
      $else
        MCLRE = INTMCLR         ; RE3 input pin enabled; MCLR disabled
        LVP = OFF              ; SINGLE-Supply ICSP disabled
        BOREN = On             ; Brown-out Reset enabled and controlled by software (SBOREN is enabled)
      $endif

      Debug = OFF               ; Background debugger disabled
      XINST = OFF               ; Extended Instruction Set and Indexed Addressing Mode disabled
      WDTCPS = WDTCPS_31       ; Divider ratio 1:65536; software control of WDTPS
      WDTE = OFF               ; WDT Disabled; SWDTEN is ignored
      WDTCWS = WDTCWS_7         ; window always open (100%); software control; keyed access not required
      WDTCCS = SC               ; Software Control
      BBSIZE = BBSIZE_512       ; Boot Block size is 512 words
      BBEN = OFF               ; Boot block disabled
      SAFEN = On               ; SAF enabled
      WRTAPP = OFF             ; Application Block not write protected
      WRTB = OFF               ; Configuration registers (300000-30000Bh) not write-protected
      WRTC = OFF               ; Boot Block (000000-0007FFh) not write-protected
      WRTD = OFF               ; Data EEPROM not write-protected
      WRTSAF = OFF             ; SAF not Write Protected
      Cp = OFF                 ; PFM and Data EEPROM code protection disabled
    Config_End
  $endif

  ' internal clock 64MHz
  $if _device = _18F27K40
  Config_Start
      FEXTOSC = OFF             ; Oscillator not enabled
      RSTOSC = HFINTOSC_64MHZ  ; HFINTOSC with HFFRQ = 64 MHz and CDIV = 1:1
      CLKOUTEN = OFF           ; CLKOUT function is disabled
      CSWEN = On               ; Writing to NOSC and NDIV is allowed
      FCMEN = OFF               ; Fail-Safe Clock Monitor disabled
      PWRTE = On               ; Power up timer enabled
      LPBOREN = On             ; ULPBOR enabled
      BORV = VBOR_285           ; Brown-out Reset Voltage (VBOR) set to 2.85V
      ZCD = OFF                 ; ZCD disabled. ZCD can be enabled by setting the ZCDSEN bit of ZCDCON
      PPS1WAY = OFF            ; PRLOCK Bit can be Set And cleared repeatedly
      STVREN = On               ; Stack full/underflow will cause Reset
      Debug = OFF               ; Background debugger disabled
      XINST = OFF               ; Extended Instruction Set and Indexed Addressing Mode disabled
      WDTCPS = WDTCPS_31       ; Divider ratio 1:65536; software control of WDTPS
      WDTE = OFF               ; WDT Disabled
      WDTCWS = WDTCWS_7         ; window always open (100%); software control; keyed access not required
      WDTCCS = SC               ; Software Control
      WRT0 = OFF               ; Block 0 (000800-003FFFh) not write-protected
      WRT1 = OFF               ; Block 1 (004000-007FFFh) not write-protected
      WRT2 = OFF               ; Block 2 (008000-00BFFFh) not write-protected
      WRT3 = OFF               ; Block 3 (00C000-00FFFFh) not write-protected
      WRT4 = OFF               ; Block 4 (010000-013FFFh) not write-protected
      WRT5 = OFF               ; Block 5 (014000-017FFFh) not write-protected
      WRT6 = OFF               ; Block 6 (018000-01BFFFh) not write-protected
      WRT7 = OFF               ; Block 7 (01C000-01FFFFh) not write-protected
      WRTC = OFF               ; Configuration registers (300000-30000Bh) not write-protected
      WRTB = OFF               ; Boot Block (000000-0007FFh) not write-protected
      WRTD = OFF               ; Data EEPROM not write-protected
      SCANE = OFF               ; Scanner module is NOT available for use, SCANMD bit is ignored

      $if Use_LVP = True
        MCLRE = EXTMCLR         ; MCLR pin enabled, RE3 input pin disabled
        LVP = On               ; SINGLE-Supply ICSP enabled
        BOREN = OFF             ; Brown-out Reset disabled in hardware and software
      $else
        MCLRE = INTMCLR         ; RE3 input pin enabled; MCLR disabled
        LVP = OFF              ; SINGLE-Supply ICSP disabled
        BOREN = On             ; Brown-out Reset enabled and controlled by software (SBOREN is enabled)
      $endif
      Cp = OFF                 ; UserNVM code protection disabled
      CPD = OFF                 ; DataNVM code protection disabled
      EBTR0 = OFF               ; Block 0 (000800-003FFFh) not protected from table reads executed in other blocks
      EBTR1 = OFF               ; Block 1 (004000-007FFFh) not protected from table reads executed in other blocks
      EBTR2 = OFF               ; Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks
      EBTR3 = OFF               ; Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks
      EBTR4 = OFF               ; Block 4 (010000-013FFFh) not protected from table reads executed in other blocks
      EBTR5 = OFF               ; Block 5 (014000-017FFFh) not protected from table reads executed in other blocks
      EBTR6 = OFF               ; Block 6 (018000-01BFFFh) not protected from table reads executed in other blocks
      EBTR7 = OFF               ; Block 7 (01C000-01FFFFh) not protected from table reads executed in other blocks
      EBTRB = OFF               ; Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
    Config_End
  $endif

  Stop
  Org ((($>>4)+1)<<4)
  HexTag: CData 0, 0, 0, 0, 0, 0, (VersionHigh << 4) | (VersionLow), 0, 0, 0, 0, 0, $55, $CC

'*****************************************************************
'end of SnapShotDemo.bas
